=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2015-11-14 17:14:26 +0000
+++ CMakeLists.txt	2016-01-25 23:45:18 +0000
@@ -35,9 +35,9 @@
     # This can not be tested using cross-compile methods with MingW and Wine.
     set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -std=c11 -mthreads -Wall -gdwarf-2 -pthread")
 
-    set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -std=c11 -mthreads -Wall -g -gdwarf-2 -O0")
+    set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -std=c11 -mthreads -Wall -g -O0")
 
-    set (CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -std=c11 -mthreads -Wall -gdwarf-2 ")
+    set (CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -std=c11 -mthreads -Wall ")
   elseif (MSVC)
     message ("MSVC is not supported, Yet. Patches welcome!")
   endif ()
@@ -45,7 +45,7 @@
   add_definitions(-D_POSIX_C_SOURCE=199309L -D_GNU_SOURCE)
   # We need -rdynamic to get the traceback information on *nix.
   # This may not work on all non-windows targets. FIXME.
-  set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -pthread -std=c11  -Wall -rdynamic -fstack-protector-strong")
+  set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -pthread -std=c11  -Wall -rdynamic -g -fstack-protector-strong")
 
   set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -pthread -std=c11 -Wall -g -rdynamic -fstack-protector-strong")
 

=== modified file 'libssts/Callback.c'
--- libssts/Callback.c	2015-04-09 18:45:12 +0000
+++ libssts/Callback.c	2016-01-26 12:42:50 +0000
@@ -22,4 +22,56 @@
 
 #include "Callback.h"
 
+SCallbackEntry *
+s_callback_entry_new_real (const schar * name,
+                           Callback callback,
+                           SCallbackType type) {
+  SCallbackEntry * self = malloc (sizeof (SCallbackEntry));
+  self->name = s_string_new (name);
+  self->callback = callback;
+  self->type = type;
+  return self;
+}
+
+void
+s_callback_entry_free (SCallbackEntry * self) {
+  free (self->name);
+  free (self);
+}
+
+void
+s_object_install_callbacks (SObject * obj,
+                            size_t n_callbacks,
+                            SCallbackEntry ** callback_entries) {
+  assert (obj);
+  assert (callback_entries);
+  s_dbg_print ("Adding Callbacks, n_callbacks: %zd", n_callbacks);
+  for (size_t i = 0; i < n_callbacks ; i++) {
+    s_dbg_print ("Loop: Adding Callbacks");
+    SCallbackEntry * entry = callback_entries[i];
+    s_object_install_callback (obj, entry);
+  }
+}
+
+void
+s_object_install_callback (SObject * obj,
+                           SCallbackEntry * callback_entry) {
+  //
+  assert (obj);
+  assert (callback_entry);
+  s_dbg_print ("Adding callback");
+  s_map_add (obj->callbacks, s_string_new (callback_entry->name), callback_entry);
+}
+
+spointer
+s_object_notify (SObject * self,
+                 schar * name,
+                 spointer * user_data) {
+  //
+  SCallbackEntry * entry = s_map_get (self->callbacks, name);
+  assert (s_string_is_equal (entry->name, name));
+
+  Callback callback = entry->callback;
+  return callback (self, user_data);
+}
 

=== modified file 'libssts/Callback.h'
--- libssts/Callback.h	2015-11-14 17:14:26 +0000
+++ libssts/Callback.h	2016-01-25 21:52:28 +0000
@@ -35,7 +35,9 @@
 /**
  * A list containing the names of the callback types.
  */
-static char * SCallbackTypeNames[] __attribute__((unused)) = {
+S_UNUSED
+static char *
+SCallbackTypeNames[] = {
   "NULL",
   "CALLBACK",
   "NOTIFY",
@@ -56,7 +58,7 @@
 };
 s_object_install_callbacks (S_OBJECT(Foo), &foo_callback_entries);
  * @endcode
- * 
+ *
  * The callback must have the following signature:
  * @code{.c}
 spointer
@@ -67,12 +69,23 @@
  @endcode
  */
 typedef struct SCallbackEntry {
-  char * name; /**< The name that is used to invoke the callback. */
+  schar * name; /**< The name that is used to invoke the callback. */
   Callback callback; /**< The callback that will be invoked.*/
   SCallbackType type; /**< The type of the callback, is it a standard callback
                        * or a notify? */
 } SCallbackEntry;
 
+#define s_callback_entry_new(n, c, t)\
+ s_callback_entry_new_real (n, CALLBACK (c), t)
+
+SCallbackEntry *
+s_callback_entry_new_real (const schar * name,
+			   Callback callback,
+			   SCallbackType type);
+
+void
+s_callback_entry_free (SCallbackEntry * entry);
+
 /**
  * @}
  * @addtogroup SObject
@@ -81,12 +94,14 @@
 
 /**
  * Installs an array of callbackentries into an SObject.
- * 
+ *
  * @param obj The object to install the callback entries into.
  * @param callbackentries an array containing the callback entries.
  */
 void
-s_object_install_callbacks (SObject * obj, SCallbackEntry ** callcackentries);
+s_object_install_callbacks (SObject * obj,
+			    size_t n_callbacks,
+			    SCallbackEntry ** callback_entries);
 
 /**
  * Installs a singel callback into an SObject.
@@ -95,7 +110,7 @@
  * @param callbackentry The callback entry to install.
  */
 void
-s_object_install_callback (SObject * obj, SCallbackEntry * callbackentry);
+s_object_install_callback (SObject * obj, SCallbackEntry * callback_entry);
 
 /**
  * @breif Do a callback.

=== modified file 'libssts/DynamicArray.c'
--- libssts/DynamicArray.c	2015-10-26 18:59:48 +0000
+++ libssts/DynamicArray.c	2016-01-26 12:26:20 +0000
@@ -42,7 +42,7 @@
                              spointer data);
 
 SDynamicArray *
-_s_dynamic_array_new_full (size_t len,
+s_dynamic_array_new_full (size_t len,
                           FreeFunc free_func,
                           FuncPointer to_json,
                           FuncPointer from_json) {
@@ -70,7 +70,7 @@
 s_dynamic_array_new (size_t len,
                      FreeFunc free_func) {
 
-  SDynamicArray * self = _s_dynamic_array_new_full (len, free_func, NULL, NULL);
+  SDynamicArray * self = s_dynamic_array_new_full (len, free_func, NULL, NULL);
 
   return self;
 }
@@ -80,7 +80,7 @@
                           FreeFunc free_func,
                           FuncPointer to_json,
                           FuncPointer from_json) {
-  SDynamicArray * self = _s_dynamic_array_new_full (len, free_func, to_json, from_json);
+  SDynamicArray * self = s_dynamic_array_new_full (len, free_func, to_json, from_json);
   return self;
 }
 
@@ -88,6 +88,7 @@
 s_dynamic_array_free (SDynamicArray * self, sboolean free_data) {
   if (free_data) {
     if (self->free_func != NULL) {
+      s_dbg_print ("SDynamicArray: free data.");
       s_dynamic_array_for_each (self,
                                 FOREACHFUNC (_private_for_each_item_free),
                                 NULL);
@@ -100,6 +101,14 @@
 
 spointer
 s_dynamic_array_get (SDynamicArray * self, size_t index) {
+  if (index > self->max_size) {
+    s_err_print ("Index out of range:\n\tMax index: %zd requested index:%zd",
+                 self->max_size, index);
+    return NULL;
+  }
+  if (self->max_size < 1) {
+    return NULL;
+  }
   return self->array[index];
 }
 
@@ -159,9 +168,11 @@
 void
 s_dynamic_array_for_each (SDynamicArray * self, ForEachFunc func,
                              spointer data) {
+  s_dbg_print ("Size: %zd", self->max_size);
   for (int i = 0; i < self->max_size && i <= self->last_item; i++) {
     spointer item = self->array[i];
     if (item) {
+      s_dbg_print ("Item Found.");
       func (self, item, data);
     }
   }
@@ -173,6 +184,8 @@
                              spointer data) {
   s_dbg_print ("Removing item from array");
   FreeFunc func = self->free_func;
-  func (item);
+  if (func) {
+    func (item);
+  }
 }
 

=== modified file 'libssts/DynamicArray.h'
--- libssts/DynamicArray.h	2015-11-14 17:14:26 +0000
+++ libssts/DynamicArray.h	2016-01-25 21:52:28 +0000
@@ -74,6 +74,12 @@
 s_dynamic_array_new (size_t len,
                      FreeFunc free_func);
 
+SDynamicArray *
+s_dynamic_array_new_full (size_t len,
+                          FreeFunc free_func,
+                          FuncPointer to_json,
+                          FuncPointer from_json);
+
 /**
  * Same as s_dynamic_array_new() but with support for to_json and from_json
  * methods.

=== modified file 'libssts/Error.c'
--- libssts/Error.c	2015-10-26 22:04:15 +0000
+++ libssts/Error.c	2016-01-25 21:52:28 +0000
@@ -152,7 +152,7 @@
   if (_internal_s_error_domain_list) {
     s_dynamic_array_free (_internal_s_error_domain_list, TRUE);
   }
-  s_dbg_print ("Done taring down error system.");
+  s_dbg_print ("Done tearing down error system.");
 }
 
 
@@ -171,7 +171,7 @@
   }
 
   if (!(_internal_s_error_domain_list)) {
-    s_dbg_print ("Array does not exist, lets crete it!");
+    s_dbg_print ("Array does not exist, lets create it!");
     _internal_s_error_domain_list =
           s_dynamic_array_new (S_ERROR_DOMAIN_RANGE_MAX,
           FREEFUNC (_internal_s_error_domain_item_free));

=== modified file 'libssts/Func.h'
--- libssts/Func.h	2015-11-14 17:14:26 +0000
+++ libssts/Func.h	2015-11-14 17:26:42 +0000
@@ -34,7 +34,7 @@
 /** @file
  * Generic function pointer definitions and data stuctures to do with
  * functions.
- * 
+ *
  * @todo create an object type that can hold functions and be able to run them.
  */
 
@@ -51,6 +51,8 @@
 #define FREEFUNC(k) ((FreeFunc) k)
 #define CALLBACK(k) ((Callback) k)
 #define FOREACHFUNC(k) ((ForEachFunc) k)
+#define COMPFUNC(k) ((CompFunc) k)
+
 
 /*
  * Common types of function pointers:
@@ -67,9 +69,10 @@
 /**
  * A generic callback function.
  *
- * A callback function can have any signature.
+ * @param obj The object to perform the callback on.
+ * @param user_data The user data to be passed to the function.
  */
-typedef void (* Callback)(void);
+typedef spointer (* Callback)(spointer obj, spointer user_data);
 
 /**
  *
@@ -92,10 +95,10 @@
 
 /**
  * A CompFunc represents a standard comparison function.
- * 
+ *
  * @param 1 object to compare
  * @param 2 object to compare
- * 
+ *
  * compare parameter 1 with parameter 2.
  */
 typedef sboolean (* CompFunc)(void*, void*);

=== modified file 'libssts/LinkedList.c'
--- libssts/LinkedList.c	2015-10-26 18:59:48 +0000
+++ libssts/LinkedList.c	2016-01-25 23:45:18 +0000
@@ -67,31 +67,40 @@
 void
 s_linked_list_free (SLinkedList * self, sboolean free_data) {
   //First we free the items if such is specified.
+  s_dbg_print ("Freeing SLinkedList");
   if (free_data) {
     s_dbg_print ("Freeing Data in LinkedList");
     s_linked_list_for_each (self,
                             FOREACHFUNC (_internal_s_liked_list_free_item),
                             NULL);
   }
-  /* Now the "snake" will "eat" its own tail. :-)
-   * Based on this: http://stackoverflow.com/a/7025370
+  /*if we have a current item we free the list, if not, we do nothing.
    */
-  SLinkedListNode * tmp;
-  SLinkedListNode * itm = self->head;
-  while (itm != NULL) {
-    tmp = itm;
-    itm = itm->next;
-    free (tmp);
+  if (self->current) {
+    /* Now the "snake" will "eat" its own tail. :-)
+     * Based on this: http://stackoverflow.com/a/7025370
+     */
+    SLinkedListNode * tmp;
+    SLinkedListNode * itm = self->head;
+    do{
+      tmp = itm;
+      itm = itm->next;
+      free (tmp);
+    } while (itm != NULL);
+    self->head = NULL;
   }
-  self->head = NULL;
   free (self);
 }
 
 void
 internal_s_linked_list_initialize_first_item (SLinkedList * self) {
   assert (self->len == 0);
-  assert (self->head == NULL && self->tail == NULL && self->current == NULL);
-  self->head = self->tail = self->current = malloc (sizeof (SLinkedListNode));
+  assert (self->head == NULL);
+  assert (self->tail == NULL);
+  assert (self->current == NULL);
+  self->current = malloc (sizeof (SLinkedListNode));
+  self->head = self->current;
+  self->tail = self->current;
   self->head->prev = NULL;
   self->tail->next = NULL;
 }
@@ -121,14 +130,19 @@
   if (self->len  == 0) { // special case when the list is empty.
     internal_s_linked_list_initialize_first_item (self);
   } else { // general case.
+    SLinkedListNode * old_tail = self->tail;
+    
     self->tail->next       = malloc (sizeof (SLinkedListNode));
     self->tail->next->next = NULL;
     self->tail             = self->tail->next;
+    self->tail->prev       = old_tail;
   }
   self->tail->data = data;
   self->len++;
 
   assert (self->tail->next == NULL);
+  assert (self->tail != NULL);
+  assert (self->current != NULL);
 }
 
 void
@@ -136,19 +150,28 @@
   if (self->len  == 0) { // special case when the list is empty.
     internal_s_linked_list_initialize_first_item (self);
   } else { // general case.
-    self->head->prev = malloc (sizeof (SLinkedListNode));
+    SLinkedListNode * old_head = self->head;
+    
+    self->head->prev       = malloc (sizeof (SLinkedListNode));
     self->head->prev->prev = NULL;
-    self->head = self->head->prev;
+    self->head             = self->head->prev;
+    self->head->next       = old_head;
   }
   self->head->data = data;
   self->len++;
 
   assert (self->head->prev == NULL);
+  assert (self->head != NULL);
+  assert (self->current != NULL);
 }
 
 sboolean
 s_linked_list_next (SLinkedList * self) {
-  if (self->current->next) {
+  if (self->current == NULL) {
+    s_err_print ("Previded SLinkedList is NULL.");
+    return FALSE;
+  }
+  if (self->current->next != NULL) {
     self->current = self->current->next;
     return TRUE;
   } else {
@@ -160,7 +183,11 @@
 
 sboolean
 s_linked_list_prev (SLinkedList * self) {
-  if (self->current->prev) {
+  if (self->current == NULL) {
+    s_err_print ("Previded SLinkedList is NULL.");
+    return FALSE;
+  }
+  if (self->current->prev != NULL) {
     self->current = self->current->prev;
     return TRUE;
   } else {
@@ -204,12 +231,18 @@
 void
 s_linked_list_head (SLinkedList * self) {
   self->current = self->head;
+  if (self->len != 0) {
+    assert (self->current != NULL);
+  }
 }
 
 
 void
 s_linked_list_tail (SLinkedList * self) {
   self->current = self->tail;
+  if (self->len != 0) {
+    assert (self->current != NULL);
+  }
 }
 
 
@@ -220,10 +253,17 @@
 
 void
 s_linked_list_remove_current (SLinkedList * self, sboolean free_data) {
+  SLinkedListNode * current = self->current;
+  
+  if (current == NULL) {
+    assert (self->len == 0);
+    s_err_print ("Trying to remove current when there is no items in the list.");
+    return;
+  }
+  
   SLinkedListNode * prev = self->current->prev;
   SLinkedListNode * next = self->current->next;
-  SLinkedListNode * current = self->current;
-
+  
   if (free_data) {
     FreeFunc free_func = self->free_func;
     spointer data = current->data;
@@ -233,14 +273,36 @@
       free (data);
     }
   }
-
-  self->current = prev;
-
-  prev->next = next;
-  next->prev = prev;
-
+  
+  
+
+  if (prev) {
+    prev->next = next;
+  }
+  if (next) {
+    next->prev = prev;
+  }
+  
+  /*
+   * Don't.. Just don't sak.. This is the best I could do...
+   * Replace it if you can come up with a better way of doing this.
+   */
+  if (current == self->tail) {
+    self->current = self->tail->prev;
+    self->tail = self->current;
+  } 
+  if (current == self->head) {
+    self->current = self->head->next;
+    self->head = self->current;
+  } 
+  if (!(current == self->head) && !(current == self->tail)) {
+    self->current = prev;
+  }
+  
   free (current);
-
+  current = NULL;
+  
+  
   self->len--;
 }
 
@@ -255,6 +317,7 @@
 
   if(self->current != NULL) {
     do {
+      s_dbg_print ("s_map_linked_list_for_each: doing loop.");
       spointer cur_itm = s_linked_list_get_current (self);
       if (cur_itm != NULL) {
         func (self, cur_itm, user_data);

=== modified file 'libssts/Map.c'
--- libssts/Map.c	2015-10-24 19:55:57 +0000
+++ libssts/Map.c	2016-01-26 12:42:50 +0000
@@ -31,7 +31,7 @@
                           * in turn also SDynamicArrays; or in template speak:
                           * SDynamicArray<SLinkedList<SMapItem*>*>*
                           */
-  
+
   CompFunc is_equal; /*  method to check if items are equal. */
   HashFunc key_hash_func;
   FreeFunc free_key;
@@ -41,7 +41,7 @@
 void
 _s_map_internal_free_map_items_for_each (SMap * self,
                                          SMapItem * item,
-                                         sboolean * free_data);
+                                         sboolean free_data);
 
 SLinkedList *
 _s_map_internal_find_bucket (SMap * self,
@@ -52,12 +52,15 @@
                                      SLinkedList * bucket,
                                      spointer key);
 
+void
+_internal_s_map_free_buckets (SLinkedList * bucket);
+
 SMapItem *
 s_map_item_new (void * key, void * value) {
   SMapItem * self = malloc (sizeof (SMapItem));
   self->key = key;
   self->value = value;
-  
+
   return self;
 }
 
@@ -65,57 +68,60 @@
 s_map_item_free (SMapItem * self,
                  FreeFunc free_key,
                  FreeFunc free_value) {
-  free_key (self->key);
-  free_value (self->value);
+  s_dbg_print ("Freeing Item!");
+  if (free_key) {
+    s_dbg_print ("Freeing Key");
+    free_key (self->key);
+  }
+  if (free_value) {
+    s_dbg_print ("Freeing Value");
+    free_value (self->value);
+  }
   free (self);
 }
 
 
-
 SMap *
 s_map_new (CompFunc comp_func,
            HashFunc key_hash_func,
            FreeFunc free_key,
            FreeFunc free_value) {
+  s_dbg_print ("Freeing SMap");
   SMap * self = malloc (sizeof (SMap));
   self->len = 0;
-  
+
   self->is_equal = comp_func;
-  
+
   /* free_* functions need to be checked if they are null and set the pointer
    * to free or s_base_object_free ()... Have to decide which...?
    */
-  if (free_key) {
-    self->free_key = free_key;
-  } else {
-    self->free_key = FREEFUNC(free);
-  }
-  if (free_value) {
-    self->free_value = free_value;
-  } else {
-    self->free_value = FREEFUNC(free);
-  }
-  
-  
-  /* if no func is set we have to use some other metod of 
+  self->free_key = free_key;
+  self->free_value = free_value;
+  
+  /*
+   * Creating a place to store the items.
+   */
+  self->array = s_dynamic_array_new (S_MAP_DEFAULT_NUMBER_OF_BUCKETS,
+                                     FREEFUNC(_internal_s_map_free_buckets));
+  
+  /* if no func is set we have to use some other metod of
    */
   if (key_hash_func == NULL){
     self->key_hash_func = s_hash_object;
   } else {
     self->key_hash_func = key_hash_func;
   }
-  
-  self->array = s_dynamic_array_new (S_MAP_DEFAULT_NUMBER_OF_BUCKETS,
-                                     FREEFUNC(s_linked_list_free));
-  
+
   return self;
 }
 
 void
 s_map_free (SMap * self, sboolean free_data) {
+  
   s_map_for_each (self,
                   FOREACHFUNC(_s_map_internal_free_map_items_for_each),
-                  &free_data);
+                  free_data);
+  
   s_dynamic_array_free (self->array, TRUE);
   free (self);
 }
@@ -124,33 +130,38 @@
 s_map_add (SMap * self, spointer key, spointer value) {
   SDynamicArray * array = self->array;
   SMapItem * item = s_map_item_new (key, value);
-  
+
   /* We have to generate a key to use as an index in the DynamicArray. */
   HashFunc hash_func = self->key_hash_func;
   hash_t hash = hash_func (key);
   /* We need to mod it with the max size of the array. */
   hash = hash % S_MAP_DEFAULT_NUMBER_OF_BUCKETS;
-  
+
   if (self->len == 0) {
     /* Since we know that there are no items in the array we can skip the
      * check if the new place is taken. and just and an array to it.
      */
+    s_dbg_print ("Adding bucket to bucket array");
     SLinkedList * bucket = s_linked_list_new (NULL);
+    s_dbg_print ("1) Appending item to linked list");
     s_dynamic_array_set (array, hash, bucket);
     s_linked_list_append (bucket, item);
-    
   } else {
+    s_dbg_print ("SMap size in more than zero.");
     /* Figure out if the bucket exists. */
     SLinkedList * bucket = _s_map_internal_find_bucket (self, key);
-    
+
     if (bucket == NULL) {
+      s_dbg_print ("Bucket does not exist");
       bucket = s_linked_list_new (NULL);
+      s_dynamic_array_set (array, hash, bucket);
     }
-    
+
     if (_s_map_internal_find_item_in_bucket (self, bucket, key)) {
       s_warn_print ("Key already exists in SMap\n");
       return;
     }
+    s_dbg_print ("2) Appending item to linked list");
     s_linked_list_append (bucket, item);
   }
   self->len++;
@@ -158,17 +169,19 @@
 
 spointer
 s_map_get (SMap * self, spointer key) {
-  spointer ret_val = NULL;
-  
+  return s_map_get_item(self, key)->value;
+}
+
+SMapItem *
+s_map_get_item (SMap * self, spointer key) {
   SLinkedList * bucket = _s_map_internal_find_bucket (self, key);
   if (bucket) {
     SMapItem * item = _s_map_internal_find_item_in_bucket (self, bucket, key);
     if (item) {
-      ret_val = item->value;
+      return item;
     }
   }
-  
-  return ret_val;
+  return NULL;
 }
 
 void
@@ -189,10 +202,13 @@
 
 void
 s_map_for_each (SMap * self, ForEachFunc foreach_func, spointer user_data) {
+  s_dbg_print ("Running For Each on SMap");
   for (size_t i = 0; i <= s_dynamic_array_last_item (self->array); i++) {
     SLinkedList * bucket = s_dynamic_array_get (self->array, i);
     if (bucket) {
+      s_dbg_print ("Found bucket");
       s_linked_list_head (bucket);
+      s_dbg_print ("No of items in list: %zd.", s_linked_list_len (bucket));
       do {
         SMapItem * item = s_linked_list_get_current (bucket);
         foreach_func (self, item, user_data);
@@ -204,13 +220,14 @@
 SLinkedList *
 _s_map_internal_find_bucket (SMap * self,
                              spointer key) {
+  s_dbg_print ("Looking for bucket");
   SLinkedList * ret_val = NULL;
-  
+
   HashFunc hash_func = self->key_hash_func;
-  hash_t hash = hash_func (key);
-  
+  hash_t hash = hash_func (key)  % S_MAP_DEFAULT_NUMBER_OF_BUCKETS;
+
   ret_val = s_dynamic_array_get (self->array, hash);
-  
+
   return ret_val;
 }
 
@@ -219,36 +236,35 @@
 _s_map_internal_find_item_in_bucket (SMap * self,
                                      SLinkedList * bucket,
                                      spointer key) {
+  s_dbg_print ("Looking for item in bucket");
   s_linked_list_head (bucket);
   do {
     SMapItem * item = SMAPITEM (s_linked_list_get_current (bucket));
+
     if (item && self->is_equal (item->key, key)) {
+      s_dbg_print ("Found Item in Bucket.");
       return item;
     }
   } while (s_linked_list_next (bucket));
-  
+
   return NULL;
 }
 
-#if 0
-void
-_s_map_internal_free_array_for_each (SDynamicArray * obj,
-                                     SLinkedList * item,
-                                     SMap * self) {
-  s_linked_list_for_each (item,
-                          FOREACHFUNC(_s_map_internal_free_bucket_for_each),
-                          self);
-  s_linked_list_free (item, FALSE);
-}
-#endif
-
 void
 _s_map_internal_free_map_items_for_each (SMap * map,
                           SMapItem * item,
-                          sboolean * free_data) {
-  if ((*free_data)) {
+                          sboolean free_data) {
+  s_dbg_print ("runnnig: _s_map_internal_free_map_items_for_each");
+  if (free_data) {
+    s_dbg_print ("Freeing K:V pair");
     s_map_item_free (item, map->free_key, map->free_value);
   } else {
     free (item);
   }
 }
+
+void
+_internal_s_map_free_buckets (SLinkedList * bucket) {
+  s_linked_list_free (bucket, FALSE);
+}
+

=== modified file 'libssts/Map.h'
--- libssts/Map.h	2015-11-14 17:14:26 +0000
+++ libssts/Map.h	2016-01-26 12:26:20 +0000
@@ -125,9 +125,9 @@
  */
 SMap *
 s_map_new (CompFunc comp_func,
-                  HashFunc key_hash_func,
-                  FreeFunc free_key,
-                  FreeFunc free_value);
+           HashFunc key_hash_func,
+           FreeFunc free_key,
+           FreeFunc free_value);
 
 /** @breif
  * This function frees an instance of an SMap.
@@ -159,6 +159,10 @@
 spointer
 s_map_get (SMap * self, spointer key);
 
+
+SMapItem *
+s_map_get_item (SMap * self, spointer key);
+
 /**
  * This function removes an item from 
  */

=== modified file 'libssts/SimpleTypeSystem.h'
--- libssts/SimpleTypeSystem.h	2015-11-14 17:14:26 +0000
+++ libssts/SimpleTypeSystem.h	2016-01-25 21:52:28 +0000
@@ -48,6 +48,7 @@
 #include "Callback.h"
 #include "defs.h"
 #include "DynamicArray.h"
+#include "LinkedList.h"
 #include "Error.h"
 #include "Func.h"
 #include "hash.h"

=== modified file 'libssts/Thread.c'
--- libssts/Thread.c	2015-10-01 10:46:50 +0000
+++ libssts/Thread.c	2015-10-28 14:45:22 +0000
@@ -1,21 +1,5 @@
 
 #include "Thread.h"
-#if 0
-  // We get this from src/external/threads.h now.
-#if __WIN32__ || __WIN64__
-  #if (NTDDI_VERSION >= NTDDI_WIN8)
-    #include <Synchapi.h>
-  #else
-    #include <WinBase.h>
-  #endif
-#else
-  /* To get nanosleep () we must define __USE_POSIX199309 for som reason.
-   */
-  #define __USE_POSIX199309
-  #include <time.h>
-  #undef __USE_POSIX199309
-#endif
-#endif
 
 /*
   Utility functions associated with threads.
@@ -27,7 +11,7 @@
   Sleep (us / 1000)
 #else
   /* We are not a windows system, so lets assume that we have posix's nanosleep
-   * Taken from: 
+   * Taken from:
    */
   struct timespec req = {0};
   req.tv_sec = (int)(us / 1000);
@@ -50,11 +34,11 @@
 SMutex *
 s_mutex_new () {
   SMutex * self = malloc (sizeof (SMutex));
-  
+
   atomic_init(&(self->locked), FALSE);
-  
+
   mtx_init (&(self->mutex), mtx_plain);
-  
+
   return self;
 }
 
@@ -95,18 +79,18 @@
 struct SThread {
   thrd_t thread;
   RunFunc func;
-  
+
   _Atomic(sboolean) is_running;
 };
 
 SThread *
 s_thread_new (RunFunc func) {
   SThread * self = malloc (sizeof (SThread));
-  
+
   self->func = func;
-  
+
   atomic_store (&(self->is_running), FALSE);
-  
+
   return self;
 }
 

=== modified file 'libssts/baseobject.c'
--- libssts/baseobject.c	2015-10-26 16:45:37 +0000
+++ libssts/baseobject.c	2015-10-28 14:45:22 +0000
@@ -97,6 +97,11 @@
 
   self->refcount = 1;
 
+  self->callbacks = s_map_new (COMPFUNC (s_string_is_equal),
+                               HASH_FUNC (sdbm_hash),
+                               FREEFUNC (free),
+                               FREEFUNC (s_callback_entry_free));
+
   s_object_set_free_method (self, method_base_free);
   s_object_set_ref_method (self, method_base_ref);
   s_object_set_unref_method (self, method_base_unref);
@@ -119,6 +124,7 @@
 void
 s_object_free (SObject * self) {
   free (self->name);
+  s_map_free (self->callbacks, TRUE);
   SObjectClass * klass = s_object_get_class (self);
   klass->free (self);
 }
@@ -193,7 +199,8 @@
 
 char *
 method_base_to_string (SObject * self) {
-  char * ret_string = s_string_new_fmt ("(SObject, References: %d)", self->base_class->get_refcount(self));
+  char * ret_string = s_string_new_fmt ("(SObject, References: %d)",
+                                        self->base_class->get_refcount(self));
   return ret_string;
 }
 

=== modified file 'libssts/hash.h'
--- libssts/hash.h	2015-11-14 17:14:26 +0000
+++ libssts/hash.h	2016-01-26 12:26:20 +0000
@@ -38,8 +38,7 @@
 /**
  * what hash function s_hash is defined to.
  */
-#define s_hash(k) sdbm_hash(k)
-
+#define s_hash sdbm_hash
 
 S_END_DECLS
 

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2015-04-10 21:36:10 +0000
+++ tests/CMakeLists.txt	2016-01-26 12:26:20 +0000
@@ -9,6 +9,14 @@
   to_string.c
   dynamic_array.h
   dynamic_array.c
+  linked_list.h
+  linked_list.c
+  map.h
+  map.c
+  callback.h
+  callback.c
+  thread.h
+  thread.c
 )
 
 add_executable (ssts_test ${TEST_FILES})

=== added file 'tests/callback.c'
--- tests/callback.c	1970-01-01 00:00:00 +0000
+++ tests/callback.c	2016-01-26 12:42:50 +0000
@@ -0,0 +1,45 @@
+#include "../libssts/SimpleTypeSystem.h"
+#include "callback.h"
+#include "test_macros.h"
+
+int
+test_callback_test_func1 (SObject * obj, spointer user_data) {
+  return 2;
+}
+
+spointer
+test_callback_test_func2 (SObject * obj, spointer user_data) {
+
+  return NULL;
+}
+
+enum {
+  TEST_CALLBACK1,
+  TEST_CALLBACK2,
+  TEST_CALLBACK_END
+};
+
+static SCallbackEntry * callback_entries[] = {NULL,};
+
+int
+test_callback (void) {
+  setup_unit ();
+
+  SObject * obj = s_object_new ();
+
+  callback_entries[TEST_CALLBACK1] =
+    s_callback_entry_new ("test1",
+                          test_callback_test_func1,
+                          S_CALLBACK_CALLBACK);
+
+  callback_entries[TEST_CALLBACK2] =
+    s_callback_entry_new ("test2",
+                          test_callback_test_func2,
+                          S_CALLBACK_CALLBACK);
+
+  s_object_install_callbacks (obj, TEST_CALLBACK_END, callback_entries);
+
+  s_object_free (obj);
+
+  end_unit ();
+}

=== added file 'tests/callback.h'
--- tests/callback.h	1970-01-01 00:00:00 +0000
+++ tests/callback.h	2015-10-28 14:45:22 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_CALLBACK__
+#define __H_TEST_CALLBACK__
+int test_callback (void);
+#endif

=== modified file 'tests/dynamic_array.c'
--- tests/dynamic_array.c	2015-10-25 19:56:39 +0000
+++ tests/dynamic_array.c	2016-01-25 21:52:28 +0000
@@ -4,7 +4,8 @@
 #include <stdint.h>
 #include <time.h>
 
-void for_each_print (SDynamicArray * self, sint item, sint * itt) {
+void
+for_each_print_arr (SDynamicArray * self, sint item, sint * itt) {
   s_print ("The number is: %d on iteration: %d\n", item, (*itt));
   (*itt)++;
 }
@@ -50,7 +51,7 @@
 
   sint itt = 0;
   s_print ("Printing array using a for-each function.\n");
-  s_dynamic_array_for_each (array, (ForEachFunc)for_each_print, &itt);
+  s_dynamic_array_for_each (array, FOREACHFUNC (for_each_print_arr), &itt);
 
   s_dynamic_array_free (array, FALSE);
 

=== modified file 'tests/error.c'
--- tests/error.c	2015-10-26 18:59:48 +0000
+++ tests/error.c	2016-01-25 21:52:28 +0000
@@ -4,7 +4,7 @@
 
 void
 _internal_print_error (SDynamicArray * arr, schar * str, spointer data) {
-    s_print ("YO!");
+  s_print ("YO!");
   s_print ("(Error Test): %s\n", str);
   s_print ("YO!");
 }
@@ -15,11 +15,9 @@
   SDynamicArray * arr = NULL;
 
   SError * e = NULL;
-  schar * s = NULL;
 
   test_case (s_error_has_error (e) == FALSE, "The error is not set. (NULL)");
 
-  schar * expected_string = NULL;
 
   e = s_error_new ();
   test_case (e != NULL, "Error is not NULL");

=== added file 'tests/linked_list.c'
--- tests/linked_list.c	1970-01-01 00:00:00 +0000
+++ tests/linked_list.c	2016-01-25 23:45:18 +0000
@@ -0,0 +1,178 @@
+#include "SimpleTypeSystem.h"
+#include "test_macros.h"
+#include <stdlib.h>
+#include <stdint.h>
+#include <time.h>
+
+void
+for_each_print_lst (SLinkedList * self, sint item, sint * itt) {
+  s_print ("The number is: %d on iteration: %d\n", item, (*itt));
+  (*itt)++;
+}
+
+int test_linked_list (void) {
+  setup_unit ();
+  
+  SLinkedList * list = NULL;
+  list = s_linked_list_new (NULL);
+  
+  test_case (list != NULL, "Linked list not null");
+  
+  s_print ("Adding a random number to the linked list (bettween 0 and 20).\n");
+  
+  srand(time(NULL));
+  
+  sint r_p = rand () % 20;
+  
+  s_print (YELLOW
+           "------------------------\n"
+           "APPEND TESTS\n"
+           "------------------------\n"
+           RESET);
+  
+  s_linked_list_append (list, (spointer)(intptr_t) r_p);
+  
+  test_case (s_linked_list_len (list) == 1, "The reported length of the linked"
+                                            " list is 1.");
+  
+  test_case (s_linked_list_get_next (list) == NULL, "There are no next items in the"
+                                             " linked list.");
+  
+  test_case (s_linked_list_get_prev (list) == NULL, "There are no previous items in"
+                                             " the linked list.");
+  
+  test_case (s_linked_list_get_current (list) != NULL, "The current item exists.");
+  
+  test_case ((intptr_t) s_linked_list_get_current (list) == r_p, "The value"
+                                 "addad was the same as the one retrieved.");
+  
+  s_print ("Removing current item in preperation for next test.\n");
+  
+  s_linked_list_remove_current (list, FALSE);
+  
+  s_print ("Item removed\n");
+  
+  test_case (s_linked_list_len (list) == 0, "The reported list length is 0");
+  
+  
+  s_print ("Filling list with twenty items (0, 1, ... 18, 19)\n");
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_append (list, (spointer)(intptr_t) i);
+  }
+  
+  
+  test_case (s_linked_list_len (list) == 20, "The reported length of the list"
+                                             " is 20.");
+  
+  s_print ("Will now print list using a callback.\n");
+  
+  sint itt = 0;
+  
+  s_linked_list_for_each (list, FOREACHFUNC (for_each_print_lst), &itt);
+  
+  s_print (YELLOW "Removing all items, from head.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_head (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print ("Filling list with twenty items (19, 18, ... 1, 0)\n");
+  
+  for (sint i = 19; i >= 0; i--) {
+    s_linked_list_append (list, (spointer)(intptr_t) i);
+  }
+  
+  s_print (YELLOW "Removing all items, from tail.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_tail (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print (YELLOW
+           "------------------------\n"
+           "PREPEND TESTS\n"
+           "------------------------\n"
+           RESET);
+
+  s_linked_list_prepend (list, (spointer)(intptr_t) r_p);
+  
+  test_case (s_linked_list_len (list) == 1, "The reported length of the linked"
+                                            " list is 1.");
+  
+  test_case (s_linked_list_get_next (list) == NULL, "There are no next items in the"
+                                             " linked list.");
+  
+  test_case (s_linked_list_get_prev (list) == NULL, "There are no previous items in"
+                                             " the linked list.");
+  
+  test_case (s_linked_list_get_current (list) != NULL, "The current item exists.");
+  
+  test_case ((intptr_t) s_linked_list_get_current (list) == r_p, "The value"
+                                 "addad was the same as the one retrieved.");
+  
+  s_print ("Removing current item in preperation for next test.\n");
+  
+  s_linked_list_remove_current (list, FALSE);
+  
+  s_print ("Item removed\n");
+  
+  test_case (s_linked_list_len (list) == 0, "The reported list length is 0");
+  
+  
+  s_print ("Filling list with twenty items (0, 1, ... 18, 19)\n");
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_prepend (list, (spointer)(intptr_t) i);
+  }
+  
+  
+  test_case (s_linked_list_len (list) == 20, "The reported length of the list"
+                                             " is 20.");
+  
+  s_print ("Will now print list using a callback.\n");
+  
+  itt = 0;
+  
+  s_linked_list_for_each (list, FOREACHFUNC (for_each_print_lst), &itt);
+  
+  s_print (YELLOW "Removing all items, from head.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_head (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_print ("Filling list with twenty items (19, 18, ... 1, 0)\n");
+  
+  for (sint i = 19; i >= 0; i--) {
+    s_linked_list_prepend (list, (spointer)(intptr_t) i);
+  }
+  
+  s_print (YELLOW "Removing all items, from tail.\n" RESET);
+  
+  for (sint i = 0; i < 20; i++) {
+    s_linked_list_tail (list);
+    s_linked_list_remove_current (list, FALSE);
+  }
+  
+  test_case (s_linked_list_len (list) == 0, "The reported length of the list"
+                                            " is 0.");
+  
+  s_linked_list_free (list, FALSE);
+  
+  s_print ("Freed stuffs.\n");
+  
+  end_unit ();
+}

=== added file 'tests/linked_list.h'
--- tests/linked_list.h	1970-01-01 00:00:00 +0000
+++ tests/linked_list.h	2016-01-25 21:52:28 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_LINKED_LIST__
+#define __H_TEST_LINKED_LIST__
+int test_linked_list (void);
+#endif

=== modified file 'tests/main.c'
--- tests/main.c	2015-04-10 21:36:10 +0000
+++ tests/main.c	2016-01-26 12:42:50 +0000
@@ -9,15 +9,22 @@
 #include "to_string.h"
 #include "error.h"
 #include "dynamic_array.h"
-
+#include "callback.h"
+#include "linked_list.h"
+#include "thread.h"
+#include "map.h"
 
 int main  (int argc, char ** argv, char ** argp) {
-  setup_suite("SSTS");
+  setup_suite ("SSTS");
   
   test_unit (test_refcount, "Refcount");
   test_unit (test_to_string, "To String");
   test_unit (test_error, "Error");
   test_unit (test_dynamic_array, "Dynamic Array");
-  
-  end_suite();
+  test_unit (test_linked_list, "Linked List");
+  test_unit (test_map, "Map");
+  test_unit (test_callback, "Callback");
+  //test_unit (test_thread, "Threading");
+
+  end_suite ();
 }

=== added file 'tests/map.c'
--- tests/map.c	1970-01-01 00:00:00 +0000
+++ tests/map.c	2016-01-26 12:26:20 +0000
@@ -0,0 +1,38 @@
+#include "SimpleTypeSystem.h"
+#include "test_macros.h"
+#include <stdlib.h>
+#include <stdint.h>
+#include <time.h>
+
+void
+for_each_print_map (SLinkedList * self, SMapItem * item, sint * itt) {
+  s_print ("Key: \"%s\" Value: %d on iteration: %d\n", (schar *)item->key,
+                                                       (sint)item->value,
+                                                       (*itt));
+  (*itt)++;
+}
+
+int test_map (void) {
+  setup_unit ();
+  
+  SMap * map = NULL;
+  map =  s_map_new (COMPFUNC (s_string_is_equal),
+                    HASH_FUNC (s_hash),
+                    NULL,
+                    NULL);
+  
+  test_case (map != NULL, "Map is not null.");
+  
+  s_map_add (map, "Hello", (spointer)(sint) 1337);
+  
+  test_case ((s_map_get (map, "Hello") == (sint) 1337), "The got value is the"
+                                                  " same as the one added to"
+                                                  " the map");
+  SMapItem * item = s_map_get_item (map, "Hello");
+  s_print ("The key was: %s, the Value was: %d\n", (schar *) item->key,
+                                                   (sint) item->value);
+  
+  s_map_free (map, TRUE);
+  
+  end_unit ();
+}

=== added file 'tests/map.h'
--- tests/map.h	1970-01-01 00:00:00 +0000
+++ tests/map.h	2016-01-26 12:26:20 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_MAP__
+#define __H_TEST_MAP__
+int test_map (void);
+#endif

=== added file 'tests/thread.c'
--- tests/thread.c	1970-01-01 00:00:00 +0000
+++ tests/thread.c	2015-10-28 14:45:22 +0000
@@ -0,0 +1,13 @@
+#include "test_macros.h"
+#include "thread.h"
+
+
+
+int
+test_thread (void) {
+  setup_unit ()
+
+
+
+  end_unit ()
+}

=== added file 'tests/thread.h'
--- tests/thread.h	1970-01-01 00:00:00 +0000
+++ tests/thread.h	2015-10-28 14:45:22 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_THREAD__
+#define __H_TEST_THREAD__
+int test_thread (void);
+#endif /*__H_TEST_THREAD__*/

